// hamamatsu linear array driver
// mp 2007; added synchronizer for discrimination ff

module ccd(ckin,trin,gin,ave,ckccd,trccd,gout,sample,syncin,syncout);

input ckin,trin,gin;
input [1:0] ave;         
input syncin;

output ckccd,trccd,gout;
output sample;
output syncout;

//gain in out: just a buffer
assign gout=gin;

// Pin assignmets
// ckin     = 43
// trin     =  2
// gin      =  4
// ave[0]   =  5
// ave[1]   =  6
// ckccd    = 20
// trccd    = 24
// gout     = 26
// sample   = 11
// syncin   = 41
// syncout  = 40

//input clock between 3.5 and 4 MHz
//generate ccd clock and card clock at input/8
//with both phases. 
//Ccd clock and sample have separate pins so that rel. phases
//can be adjusted 

wire[7:0] dq;
wire[2:0] nq;
wire r, inttrig;
assign inttrig=~trin;

wire ckaux;

lpm_counter cnt1 (.clock(ckin),.q(nq),.eq(dq));
defparam cnt1.lpm_width = 3;

assign ckccd=nq[2];

reg tr0,tr1;

//trigger: 
wire p1,p2;
assign p1=dq[2];
assign p2=dq[6];

always@(negedge r or negedge tr1)
	if(!tr1) 
		tr0=1'b0;
	else 
		tr0=1'b1;

always@(posedge p2 or posedge p1)
	if(p1) 
		tr1=1'b1;
	else 
		if(tr0) tr1=1'b0;

assign trccd=tr1;

//average counter 3=no average    2=2 averages
//               1=5 averages  
// 0=10 averages
// 1=5 averages
// 2=2 averages
// 3=0 averages
// but numbers can be changed below 

`define ave0  99 
`define ave1  49
`define ave2  19
`define avewidth 7

wire [`avewidth-1:0] aq;
wire [3:0] dt;
wire v2,v1,v0;

lpm_counter cnt2 (.clock(~inttrig),.q(aq),.aclr(!tr1));
defparam cnt2.lpm_width = `avewidth;

lpm_compare cmp1 (.dataa(aq),.datab(`ave0),.aeb(v0));
defparam cmp1.lpm_width = `avewidth;
defparam cmp1.one_input_is_constant = "YES";

lpm_compare cmp2 (.dataa(aq),.datab(`ave1),.aeb(v1));
defparam cmp2.lpm_width = `avewidth;
defparam cmp2.one_input_is_constant = "YES";

lpm_compare cmp3 (.dataa(aq),.datab(`ave2),.aeb(v2));
defparam cmp3.lpm_width = `avewidth;
defparam cmp3.one_input_is_constant = "YES";

assign dt[3]=inttrig;
assign dt[2]=v2;
assign dt[1]=v1;
assign dt[0]=v0;

mux mx(.sel(ave),.data(dt),.result(r)); 
defparam mx.width = 4; 
defparam mx.widths = 2; 

//sample
wire[9:0] q3;
wire ct3en;
reg ct3en;

always@(posedge ckin)
	if(!trccd)
		ct3en=1'b1;
	else 
		if(q3[8])
			ct3en=1'b0;

wire p3;
assign p3=dq[3];

lpm_counter cnt3 (.clock(p3),.q(q3),.aclr(!tr1),.clk_en(ct3en));
defparam cnt3.lpm_width = 9;

assign sample=p3 & ct3en;

// external discriminator sync
// syncin is sampled after 32 ccdck pulses after trin
// and stored in the jk flip-flop ffsync
// when in average mode a single 0 sets ffsync to 0 until readout
// ffsync is reset at pixel 240 during readout

wire[5:0] qau;
lpm_counter cntaux (.clock(ckccd),.q(qau),.aclr(trin),.clk_en(~qau[5]));
defparam cntaux.lpm_width = 6;

wire so;
assign syncout=so;
jkff ffsync (.j(syncin),.k(0),.clk(qau[5]),.clrn(~(q3[7] & q3[6] & q3[5])),.prn(1),.q(so));


endmodule
	


